import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as scst
import pymc3 as pm

RawData = [ ['City',     'Temperature', 'Latitude'], 
            ['Sapporo',     15,             43.1],
            ['Sendai',      20,             38.3],
            ['Niigata',     17,             37.9],
            ['Kanazawa',    18,             36.6],
            ['Tokyo',       24,             35.7], 
            ['Osaka',       22,             34.7],
            ['Fukuoka',     22,             33.6],
            ['Kochi',       24,             33.6],
            ['Kagoshima',   23,             31.6],
            ['Naha',        26,             26.2]
]

ID = []
Y = []
X = []
N = len(RawData) - 1
for i in range(N):
    ID.append(RawData[i + 1][0])
    Y.append(RawData[i + 1][1])
    X.append(RawData[i + 1][2])

for v in zip(ID, Y, X):
    print(v)

with pm.Model() as RegModel:
	b0 = pm.Uniform('b0', lower = -100.0, upper = 100.0)
	b1 = pm.Uniform('b1', lower = -100.0, upper = 100.0)
	sgm = pm.Uniform('sgm', lower = 0.0, upper = 100.0)
	muY = b0 + b1 * X

	Y_obs = pm.Normal('Y_obs', mu = muY, sd = sgm, observed = Y)
	trace = pm.sample()

	pm.traceplot(trace)
	plt.show()
	summary_trace = pm.summary(trace)
	print(summary_trace)
	f = open('summary.txt', 'w')
	f.write('Summary...\n{}'.format(summary_trace))

	b0_trace = trace['b0']
	b1_trace = trace['b1']
	r, p = scst.pearsonr(b0_trace, b1_trace)
	plt.title('r = {0:.3f}'.format(r))
	plt.xlabel('b0')
	plt.ylabel('b1')
	plt.plot(b0_trace, b1_trace, 'b.')
	plt.show()

	nPos = 0
	for v in b1_trace:
		if v > 0.0:
			nPos += 1
	rPos = nPos / len(b1_trace)
	print('Prop(b1 > 0) = ', rPos)
	f.write('\nProp(b1 > 0) = {0:.3f}\n'.format(rPos))
	

	muY_trace = []
	for i in range(N):
		muY_trace.append([])
	for t in range(len(b0_trace)):
		for i in range(N):
			muY_trace[i].append(b0_trace[t] + b1_trace[t] * X[i])
	
	MedPredY = []
	for i in range(N):
		MedPredY.append(np.percentile(muY_trace[i], 50))
	plt.plot(Y, MedPredY, 'bo')
	plt.xlabel('Data')
	plt.ylabel('Prediction')
	for nm, y, pred in zip(ID, Y, MedPredY):
		plt.text(y + 0.1, pred + 0.1, nm)
	plt.plot([14, 28], [14, 28], 'b-')
	plt.show()

	f.close()


